//============================================================
//
//	psp_video.h - PSP GU video functions
//
//============================================================

#ifndef PSP_VIDEO_H
#define PSP_VIDEO_H

#define SCR_WIDTH			480
#define SCR_HEIGHT			272
#define BUF_WIDTH			512
#define FRAMESIZE			(BUF_WIDTH * SCR_HEIGHT * sizeof(UINT16))
#define FRAMESIZE32 		(BUF_WIDTH * SCR_HEIGHT * sizeof(UINT32))

#define SLICE_SIZE			64 /* change this to experiment with different page-cache sizes */

#define MAKECOL15(r, g, b)	(((b & 0xf8) << 7) | ((g & 0xf8) << 2) | ((r & 0xf8) >> 3))
#define GETR15(col) 		(((col << 3) & 0xf8) | ((col >>  2) & 0x07))
#define GETG15(col) 		(((col >> 2) & 0xf8) | ((col >>  7) & 0x07))
#define GETB15(col) 		(((col >> 7) & 0xf8) | ((col >> 12) & 0x07))

#define MAKECOL32(r, g, b)	(0xff000000 | ((b & 0xff) << 16) | ((g & 0xff) << 8) | (r & 0xff))
#define GETR32(col) 		((col >>  0) & 0xff)
#define GETG32(col) 		((col >>  8) & 0xff)
#define GETB32(col) 		((col >> 16) & 0xff)

#define COLOR_BLACK 		  0,  0,  0
#define COLOR_RED			255,  0,  0
#define COLOR_GREEN 		  0,255,  0
#define COLOR_BLUE			  0,  0,255
#define COLOR_YELLOW		255,255,  0
#define COLOR_PURPLE		255,  0,255
#define COLOR_CYAN			  0,255,255
#define COLOR_WHITE 		255,255,255
#define COLOR_GRAY			127,127,127
#define COLOR_DARKRED		127,  0,  0
#define COLOR_DARKGREEN 	  0,127,  0
#define COLOR_DARKBLUE		  0,  0,127
#define COLOR_DARKYELLOW	127,127,  0
#define COLOR_DARKPURPLE	127,  0,127
#define COLOR_DARKCYAN		  0,127,127
#define COLOR_DARKGRAY		 63, 63, 63

#define GU_FRAME_ADDR(frame)		(UINT16 *)((UINT32)frame | 0x44000000)
#define CNVCOL15TO32(c) 			((GETB15(c) << 16) | (GETG15(c) << 8) | GETR15(c))

#define SWIZZLED_UV(tex, u, v)		&tex[((((v) & ~7) << 9) | (((u) & ~7) << 3))]
#define SWIZZLED_8x8(tex, idx)		&tex[(idx) << 6]
#define SWIZZLED_16x16(tex, idx)	&tex[((idx & ~31) << 8) | ((idx & 31) << 7)]
#define SWIZZLED_32x32(tex, idx)	&tex[((idx & ~15) << 10) | ((idx & 15) << 8)]
#define SWIZZLED_GAP				((BUF_WIDTH << 3) - 8*8)


/* 0:_J[ƂgȂD */
#define USE_VCOLOR 0

	#if (1==USE_VCOLOR)
		/* ʓIȁA_J[ƂgꍇB */
		#define TEXTURE_FLAGS		(GU_TEXTURE_16BIT | GU_COLOR_5551 | GU_VERTEX_16BIT | GU_TRANSFORM_2D)
	#else
		/* F_J[gȂꍇ́AԈႦāuGU_COLOR_5551vtƓ񂩂璍ӁB */
	//	#define TEXTURE_FLAGS		(GU_TEXTURE_16BIT | 				GU_VERTEX_16BIT | GU_TRANSFORM_2D)
		#define TEXTURE_FLAGS		(GU_TEXTURE_16BIT | 				GU_VERTEX_16BIT | GU_TRANSFORM_2D)
	#endif

struct Vertex_uvcxyz
{
	UINT16 u, v;
	#if (1==USE_VCOLOR)
	UINT16 color;
	#endif
	INT16  x, y, z;
};

typedef struct rect_t
{
	int left;
	int top;
	int right;
	int bottom;
} RECT;

#ifdef seta
	#define ADJUST_GU_MEMORY_KB (1)
	/* rezon 128kBytesȂƓȂ (;^;) */
#endif // seta

#ifndef ADJUST_GU_MEMORY_KB
	//#define ADJUST_GU_MEMORY_KB (128)
	#define ADJUST_GU_MEMORY_KB (1)
#endif // ADJUST_GU_MEMORY_KB

#define GU_MAX_MEMORY (ADJUST_GU_MEMORY_KB * 1024)

extern UINT8 gulist[GU_MAX_MEMORY];

extern void *show_frame;
extern void *draw_frame;
extern void *depth_frame;
extern void *work_frame;
extern void *tex_frame;
extern RECT full_rect;

//void psp_wait_vsync(void);
//void psp_wait_vsync_frame(int count);
void psp_flip_screen(void);//int vsync);

void psp_video_init(void);
void psp_video_exit(void);
UINT16 *psp_frame_offs(void *frame, int offs);
UINT16 *psp_frame_addr(void *frame, int x, int y);
void psp_clear_screen(void);
void psp_clear_frame(void *frame);
void psp_clear_rect(void *frame, const rectangle *cliprect);
void psp_fill_frame(void *frame, UINT32 color);
void psp_fill_rect(void *frame, UINT32 color, const rectangle *cliprect);
void psp_box_rect( void *frame, UINT16 color, const rectangle *cliprect);

void psp_copy_rect(void *src, void *dst, const rectangle *src_rect, const rectangle *dst_rect);

void psp_box_frame(void *frame, int x1, int y1, int x2, int y2, UINT16 color);
void psp_box_clear_frame(void *frame, int x1, int y1, int x2, int y2/*, UINT32 color32*/);
//void psp_box_fill(	int x1, int y1, int x2, int y2, UINT16 color);
void psp_box_fill(void *frame,	int x1, int y1, int x2, int y2, UINT16 color);




#if 1
/* inlineWJp */
//============================================================
//	psp_box_clear
//============================================================
/* Furect\́vgAR[hđB */
#define PSP_BOX_CLEAR_FRAME( frame, x1, y1, x2, y2/*, UINT32 color32*/)\
{\
	sceGuStart(GU_DIRECT, gulist);\
	sceGuDrawBufferList(GU_PSM_5551, frame, BUF_WIDTH);\
	sceGuScissor(x1, y1, x2 + 1, y2 + 1);\
	/*sceGuClearColor(psp_BG_color32);*/\
	sceGuClear(GU_COLOR_BUFFER_BIT);\
	/*sceGuClearColor(0);*/\
	sceGuFinish();\
	sceGuSync(0, 0);\
}
#endif








#endif /* PSP_VIDE_H */
